function [Simu] = MCMC_VAR(y,lags,nb_MCMC,minnesota,graph)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Tutorial 3 : Exercise One (and two)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Inputs : 
%%%%% y contains the different time series (T * k)
%%%%% lags : denotes the number of lags (for the exercise = 6)
%%%%% nb_MCMC : number of MCMC iterations 
%%%%% minnesota : if =0, Gibbs sampler under NIP; if =1, Minnesota prior;
%%%%% if =2, trick that shrinks constrained parameters of exercise two
%%%%% toward zero.
%%%%% graph : if =1, some convergence graphics are displayed at the end of
%%%%% the simulation (by default = 0)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Outputs :
%%%%% Simu is a structure containing :
%%%%% DS_beta and DS_Sigma : Direct Sampling of beta and Sigma under NIP
%%%%% post_beta and post_Sigma : MCMC Posterior draws of Beta and Sigma
%%%%% The structure also includes all the summary statistics asked in
%%%%% exercise one.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% By default, set the variable graph = 0.
if(nargin<5)
    graph=0;
end

%%%% Check if the variable y exhibits an appropriate size.
dimension = min(size(y)); %% dimension denotes the number of variables in y.
if(max(size(y))~=size(y,1))
    y = y';
end
%%%%%%%%%%%
%%%% Generate the VAR variables : 
%%%% The variable y becomes T-lags * k
%%%% X_stack is a 3 dimension matrix dimension * dimension*(1+dimension*lags) * T-lags and its
%%%% entries are assigned as follows : 
%%%% X_stack = [1 y_{1,t-1},y_{1,t-2},...,y_{1,t-lags},y_{2,t-1},...y_{2,t-lags},...,y_{n,t-lags},0,0,...,0  ; 0,0,... 0 1 y_{1,t-1},y_{1,t-2},...,y_{1,t-lags},y_{2,t-1},...y_{2,t-lags},...,y_{n,t-lags},0,0,...,0; ... ]
%%%% X_vec is a matrix (T-lags * (dimension*lags + dimension)) and exhibits
%%%% the form : X_vec(t,:) = [1
%%%% y_{1,t-1},y_{1,t-2},...,y_{1,t-lags},y_{2,t-1},...y_{2,t-lags},...,y_{n,t-lags}]
%%%%%%%%%%%
[y X_stack X_vec] = create_X(y,lags);
T = max(size(y));
dimension_beta = max(size(X_stack(:,:,1)));


prod_X_vec = X_vec'*X_vec;
Id_X_vec = eye(size(prod_X_vec,1));
inv_X_vec = (X_vec'*X_vec)\Id_X_vec;
beta_ols = inv_X_vec*X_vec'*y;
Sigma_ols = (y-X_vec*beta_ols)'*(y-X_vec*beta_ols)/(T-dimension_beta);
disp('**********************************');
disp('***** DIRECT SAMPLING ************');
disp('**********************************');
%%%%%%%%%%%
%%%% Initialisation of the Direct sampling under NIP
%%%%%%%%%%%
S = zeros(dimension,dimension);
beta_ols_vec = reshape(beta_ols,dimension_beta,1);
for t=1:T
    aide = X_stack(:,:,t)*beta_ols_vec;
    S = S +  (y(t,:)'-aide)*(y(t,:)-aide');
end
df_IW = T-dimension_beta;
Simu.DS_beta = zeros(dimension_beta,nb_MCMC);
Simu.DS_Sigma = zeros(dimension^2,nb_MCMC);
%%%%%%%%%%%
%%%% Direct sampling : the number of draws equals to nb_MCMC
%%%%%%%%%%%
for i=1:nb_MCMC
   Sigma = iwishrnd(S,df_IW);
   Sigma_star = tidy_cov_mat(kron(Sigma,inv_X_vec));
   Simu.DS_beta(:,i) = mvnrnd(beta_ols_vec,Sigma_star)';
   Simu.DS_Sigma(:,i) = reshape(Sigma,dimension^2,1);
end
disp('**********************************');
disp('***** BEGINNING OF THE MCMC ******');
disp('**********************************');
%%%%%%%%%%%
%%%% MCMC Initialisation
%%%% Sigma_bar denotes the matrix hyper-parameter of the IW distribution
%%%% df_Sigma stands for the degreee of freedom of the IW distribution
%%%% Q is the variance-covariance prior on beta
%%%% beta_0 denotes the expectation prior on beta
%%%%%%%%%%%
Sigma_bar = zeros(dimension);
df_Sigma = dimension-dimension_beta;
if(minnesota==1) %% Minnesota prior as asked in exercise one
    theta = 0.5;
    lambda = 0.2;
    sigma_scale = sqrt(diag(Sigma_ols));
    Q = zeros(dimension_beta);
    beta_0 = zeros(dimension_beta,1);
    for i=1:dimension
        ind = i+(i-1)*(2*lags); 
        Q(ind,ind) = 100;
        for j=1:dimension
            for z=1:lags
                if(i==j)
                    ind = i+(i-1)*(2*lags)+(j-1)*lags+z; 
                    Q(ind,ind) = lambda/z;
                    if(z==1)
                        beta_0(ind) = 1;
                    end
                else
                    ind = i+(i-1)*(2*lags)+(j-1)*lags+z; 
                    Q(ind,ind) = (sigma_scale(i)/sigma_scale(j))*theta*lambda/z;
                end
            end
        end
    end
    inv_Q = inv(Q);
    Cov_prior = Q\beta_0;    
elseif(minnesota==2) %% Minnesota prior where beta_(2,i) with i=1,...,lags have a variance prior close to zero in order to shrink them toward zero
    theta = 0.5;
    lambda = 0.2;
    sigma_scale = sqrt(diag(Sigma_ols));
    Q = zeros(dimension_beta);
    beta_0 = zeros(dimension_beta,1);
    for i=1:dimension
        ind = i+(i-1)*(2*lags); 
        Q(ind,ind) = 100;
        for j=1:dimension
            for z=1:lags
                if(i==j)
                    ind = i+(i-1)*(2*lags)+(j-1)*lags+z; 
                    Q(ind,ind) = lambda/z;
                    if(z==1)
                        beta_0(ind) = 1;
                    end
                else
                    ind = i+(i-1)*(2*lags)+(j-1)*lags+z; 
                    Q(ind,ind) = (sigma_scale(i)/sigma_scale(j))*theta*lambda/z;
                end
                if(i==2 && j~=i)
                    Q(ind,ind) = 1e-8; %% Shrinkage to zero
                end
            end
        end
    end
    inv_Q = inv(Q);
    Cov_prior = Q\beta_0;    
else
    inv_Q = zeros(dimension_beta); %% NIP prior on beta (inv_Q is the inverse of the variance-covariance matrix prior of beta
end
Sigma = Sigma_ols;
Sigma_inv = inv(Sigma_ols);
beta = zeros(dimension_beta,1);
burn_in = ceil(0.3*nb_MCMC);
nb_post = nb_MCMC-burn_in;
beta_post = zeros(dimension_beta,nb_post);
Sigma_post = zeros(dimension^2,nb_post);
sum_beta = zeros(4,nb_post);
eigen_value = zeros(nb_post,1);
%%%%%%%%%%%
%%%% MCMC iterations
%%%%%%%%%%%
iter = 1;
for i=1:nb_MCMC
    progressbar(i/nb_MCMC);
    %%%%%%%%%%%
    %%%% Saving posterior samples
    %%%%%%%%%%%
    if(i>burn_in)
        beta_post(:,iter) = beta;
        Sigma_post(:,iter) = reshape(Sigma,dimension^2,1);
        sum_beta(1,iter) = sum(beta(2:lags+1));
        sum_beta(2,iter) = sum(beta(lags+2:1+2*lags));
        sum_beta(3,iter) = sum(beta(2*lags+3:3*lags+3));
        sum_beta(4,iter) = sum(beta(3*lags+4:end));
                
        vect_beta = [beta(2:1+2*lags);beta(3+2*lags:end)];
        A = zeros(dimension,dimension);
        for z=1:lags
            A = A + [vect_beta(z) vect_beta(z+lags);vect_beta(z+2*lags) vect_beta(z+3*lags)];
        end
        A = eye(dimension)-A;
        eigen_value(iter) = min(eig(A'*A));
        iter = iter +1;
    end
    %%%%%%%%%%%%%
    %%% Update Beta|Sigma
    %%%%%%%%%%%%%
   if(minnesota>0)      
       kron_Sigma = kron(Sigma_inv,prod_X_vec);
       M_star = kron_Sigma + inv_Q;
       inv_M_star = inv(M_star);
       inv_M_star = tidy_cov_mat(inv_M_star);
       vec_beta_star = inv_M_star*(kron_Sigma*beta_ols_vec + Cov_prior);
       beta = mvnrnd(vec_beta_star,inv_M_star)';
   else
       Sigma_star = tidy_cov_mat(kron(Sigma,inv_X_vec));
       beta = mvnrnd(beta_ols_vec,Sigma_star)';
   end
    
    %%%%%%%%%%%%%
    %%% Update Sigma|Beta
    %%%%%%%%%%%%%
    eps = Sigma_bar;
    for t=1:T
        aide = X_stack(:,:,t)*beta;
        eps = eps +  (y(t,:)'-aide)*(y(t,:)-aide');
    end
    inv_post = inv(eps);
    Sigma_inv = wishrnd(inv_post,T+df_Sigma);
    Sigma = inv(Sigma_inv);
    
    
end
%%%%%%%%%%%%%
%%% Gathering all the statistics into the structure Simu
%%%%%%%%%%%%%

Simu.post_beta = beta_post;
Simu.post_Sigma = Sigma_post;
Simu.sum_beta = sum_beta;
Simu.eigen_value = eigen_value;
Simu.post_mean_eigen = mean(eigen_value);
Simu.post_std_eigen = std(eigen_value);
Simu.post_mean_beta = mean(beta_post,2);
Simu.post_std_beta = std(beta_post,0,2);
Simu.post_mean_sum_beta = mean(sum_beta,2);
Simu.post_std_sum_beta = std(sum_beta,0,2);

std_post = sqrt(Sigma_post);
Simu.post_mean_std = mean(std_post,2);
Simu.post_std_std = std(std_post,0,2);
Simu.post_rho = Sigma_post(2,:)./sqrt((Sigma_post(1,:).*Sigma_post(3,:)));
Simu.post_mean_rho = mean(Simu.post_rho);
Simu.post_std_rho = std(Simu.post_rho);

%%%%%%%%%%%%%
%%% Testing for the convergence by Geweke's test, Cusum plot and ACF
%%%%%%%%%%%%%
Simu.test_Geweke = Geweke_test(beta_post);
[Simu.cusum] = cusum_plot(beta_post,graph);
if(graph==1)
    figure;
    for z=1:dimension_beta
        subplot(ceil(0.25*dimension_beta),4,z),autocorr(beta_post(z,:),200),title(['Sample ACF of \beta : ' num2str(z)]);
    end
    figure;
    for z=1:dimension^2
        subplot(ceil(0.5*dimension^2),2,z),autocorr(Sigma_post(z,:),200),title(['Sample ACF of \sigma^2 : ' num2str(z)]);
    end
end